@if defined echo (echo %echo% ) else echo off
setlocal

@REM set DEBUG=yes to turn on debugging statements
set DEBUG_UPDATE=no

:Launch
set LaunchTitle=%0
Echo Starting [ %LaunchTitle% ] and writing install logs to %LaunchTitle%.log
set LOGFILENAME="%LaunchTitle%.log"

:CheckEnvironmentVariable
Echo.
IF "%DEBUG_UPDATE%" == "yes" Echo Checking Environment Variable
echo CheckEnvironmentVariable > %LOGFILENAME% 2>&1
echo IF "%TC_ROOT%" == "" goto MissingEnvVariable >> %LOGFILENAME% 2>&1
IF "%TC_ROOT%" == "" goto MissingEnvVariable
echo IF "%WAS_HOME%" == "" goto MissingEnvVariable >> %LOGFILENAME% 2>&1
IF "%WAS_HOME%" == "" goto MissingEnvVariable

:CheckAuthority
Echo.
IF "%DEBUG_UPDATE%"== "yes" Echo Checking user authority
Echo Checking user authority >> %LOGFILENAME% 2>&1
IF NOT EXIST %TC_ROOT%\tmp  mkdir %TC_ROOT%\tmp
IF %ERRORLEVEL% == 1 goto NotEnoughAuthority
IF NOT EXIST %TC_ROOT%\tmp\utils  mkdir %TC_ROOT%\tmp\utils
IF %ERRORLEVEL% == 1 goto NotEnoughAuthority

:CopyScripts
Echo.
Echo Copying Scripts >> %LOGFILENAME% 2>&1
IF "%DEBUG_UPDATE%"== "yes" Echo Copying Scripts
XCOPY /E /Y utils %TC_ROOT%\tmp\utils >> %LOGFILENAME% 2>&1

:VerifyUser
Echo.
Echo Verifying User >> %LOGFILENAME% 2>&1
IF "%DEBUG_UPDATE%" == "yes" Echo Verifying User
%TC_ROOT%\bin\bash --login -c "echo User;$TC_ROOT/tmp/utils/check_user.sh"
IF %ERRORLEVEL% == 1 goto IncorrectUser

:CheckProperties
Echo.
Echo Verifying %TC_ROOT%\home\thinkcontrol\properties\version\THINKCONTROL.product >> %LOGFILENAME% 2>&1
IF "%DEBUG_UPDATE%" == "yes" Echo Verifying %TC_ROOT%\home\thinkcontrol\properties\version\THINKCONTROL.product
IF NOT EXIST "%TC_ROOT%\home\thinkcontrol\properties\version\THINKCONTROL.product" goto CopyFiles
Echo Properties files already exist >> %LOGFILENAME% 2>&1
IF "%DEBUG_UPDATE%" == "yes" Echo Properties files already exist
goto Wizard

:CopyFiles
set TCHOME=%TC_ROOT%\home\thinkcontrol
Echo Creating directories for properties files >> %LOGFILENAME% 2>&1
IF "%DEBUG_UPDATE%" == "yes" Echo Creating directories for properties files
mkdir %TCHOME%\properties >> %LOGFILENAME% 2>&1
mkdir %TCHOME%\properties\version >> %LOGFILENAME% 2>&1
mkdir %TCHOME%\properties\version\backup >> %LOGFILENAME% 2>&1
mkdir %TCHOME%\properties\version\dtd >> %LOGFILENAME% 2>&1
mkdir %TCHOME%\properties\version\history >> %LOGFILENAME% 2>&1
mkdir %TCHOME%\properties\version\log >> %LOGFILENAME% 2>&1
mkdir %TCHOME%\properties\version\log\wizard >> %LOGFILENAME% 2>&1
Echo Copying properties files >> %LOGFILENAME% 2>&1
IF "%DEBUG_UPDATE%" == "yes" Echo XCOPY /E /Q ..\properties %TCHOME%
Echo XCOPY /E /Q ..\properties %TCHOME% >> %LOGFILENAME% 2>&1
IF "%DEBUG_UPDATE%" == "yes" (
  XCOPY /E /Y ..\properties %TCHOME%\properties  
) else (
  XCOPY /E /Y ..\properties %TCHOME%\properties >> %LOGFILENAME% 2>&1
)
IF %ERRORLEVEL% == 1 goto FailedToCopy

:Wizard
Echo.
Echo Verifying wizard installer.jar exists >> %LOGFILENAME% 2>&1
IF "%DEBUG_UPDATE%" == "yes" Echo Verifying wizard installer.jar exists
IF NOT EXIST "%~dp0installer.jar" goto FailWizard
goto ParseArgs

:ParseArgs
set PREREQ_DISABLE_INST=
set PREREQ_DISABLE_UNINST=
set FOR_EFIX=
set RUN_JAVA_HOME=C:\WebSphere\AppServer5\java
goto ValidateArgs

:ValidateArgs
IF "%2"=="-dpInstall" goto SetPrereqInstall
IF "%2"=="-dpUninstall" goto SetPrereqUninstall
IF "%2"=="-efixOnly" goto SetEFixOnly
IF "%2"=="-skipJDKCopy" goto SetExternalJDK

IF "%2"=="" goto SetupJDK

IF NOT "%2"=="-usage" Echo Usage error: [ "%2" is an unrecognized argument ]

Echo.
Echo Usage: updateWizard ([install | uninstall ][ -efixOnly | -dpInstall | -dpUninstall | -skipJDKCopy | -usage ])
Echo.
Echo   efixOnly    - Run the installer in EFix Processing mode only. 
Echo   dpInstall   - Disable install prerequisite error locking.	
Echo   dpUninstall - Disable uninstall prerequisite error locking.
Echo   skipJDKCopy - Run the installer directly from JAVA_HOME.
Echo   usage       - Display syntax help. 
Echo.
GOTO Usage

:SetEFixOnly
set FOR_EFIX=-W UpdateActionSelect.disablePTFOptions=yes
shift
GoTo ValidateArgs

:SetPrereqInstall
set PREREQ_DISABLE_INST=-W EfixInstallPrereqError.prereqOverride="yes"
shift
GoTo ValidateArgs  

:SetPrereqUninstall
set PREREQ_DISABLE_UNINST=-W EfixUninstallPrereqError.prereqOverride="yes"
shift
GoTo ValidateArgs

:SetExternalJDK
set RUN_JAVA_HOME=yes
shift
GoTo ValidateArgs

:SetupJDK
set JAVA_TMP=%~dp0java_tmp
IF NOT EXIST "%JAVA_TMP%\bin" GoTo PrepareJDK
IF NOT EXIST "%JAVA_TMP%\bin\java.exe" GoTo PrepareJDK
goto UseLocalJDK

:PrepareJDK
IF NOT "%OS%" == "Windows_NT" GoTo Win9xQuoteStrip
GoTo WinNTQuoteStrip

:Win9XQuoteStrip
@REM FOR %%A IN (%JAVA_HOME%) DO SET NEW_JAVA_HOME=%%A
@REM set JAVA_HOME=%NEW_JAVA_HOME%
@REM IF "%JAVA_HOME%" == "" GoTo CheckSetup
IF "%JAVA_HOME%" == "" GoTo CheckSetup
IF "%RUN_JAVA_HOME%" == "yes" goto RunOffJavaHome

:WinNTQuoteStrip
set JAVA_HOME_TMP=%JAVA_HOME%
set MOD_JAVA_HOME=%JAVA_HOME_TMP:"=%
set NEW_JAVA_HOME=%MOD_JAVA_HOME:"=%
set JAVA_HOME=%NEW_JAVA_HOME%
IF "%JAVA_HOME%" == "=" GoTo CheckSetup
IF "%RUN_JAVA_HOME%" == "yes" goto RunOffJavaHome
goto UseJavaHome

:UseLocalJDK
set JAVA_HOME=%JAVA_TMP%
IF "%DEBUG_UPDATE%" == "yes" Echo Using the local copy of the JDK, made previously, for processing:
IF "%DEBUG_UPDATE%" == "yes" Echo JAVA_HOME: [ %JAVA_HOME% ]
IF "%DEBUG_UPDATE%" == "yes" Echo.
goto LaunchInstaller

:RunOffJavaHome
Echo No local copy of the JDK will be created.  
Echo The installer will run directly from JAVA_HOME.
Echo.
goto LaunchInstaller

:UseJavaHome
IF "%DEBUG_UPDATE%" == "yes" Echo JAVA_HOME was set.
IF "%DEBUG_UPDATE%" == "yes" Echo JAVA_HOME: [ %JAVA_HOME% ]
IF "%DEBUG_UPDATE%" == "yes" Echo.
goto TestJDK

:CheckSetup
IF "%DEBUG_UPDATE%" == "yes" Echo JAVA_HOME is not set.
IF "%DEBUG_UPDATE%" == "yes" Echo Attempting to locate setupCmdLine.bat.
IF "%DEBUG_UPDATE%" == "yes" Echo.
IF NOT EXIST "%~dp0..\bin" GoTo FailSetup
IF NOT EXIST "%~dp0..\bin\setupCmdLine.bat" GoTo FailSetup
GoTo CallSetup

:FailSetup
Echo JAVA_HOME must be set before launching the update wizard:
Echo JAVA_HOME was not set;
Echo Set JAVA_HOME manually with the install_root/bin/setupCmdLine.bat or setupClient.bat script.
Echo.
Echo (setupCmdLine.bat or setupClient.bat is usually located beneath your
Echo WebSphere Application Server installation, in the 'bin'
Echo directory.)
Echo.
Echo Exiting.
Echo.
GOTO Usage

:CallSetup
call "%~dp0..\bin\setupCmdLine.bat"
GoTo TestJDK

:TestJDK
IF NOT EXIST "%JAVA_HOME%\bin" GoTo FailJavaHome
IF NOT EXIST "%JAVA_HOME%\bin\java.exe" GoTo FailJavaHome
IF "%RUN_JAVA_HOME%" == "yes" goto RunOffJavaHome
goto CopyJDK

:FailJavaHome
Echo The JDK was not found within the set JAVA_HOME:
Echo JAVA_HOME: [ %JAVA_HOME%\bin\java.exe ]
Echo.
Echo Exiting.
Echo.
GOTO Usage

:CopyJDK
IF NOT "%FOR_EFIX%"=="" goto SkipCopyJDK
IF "%DEBUG_UPDATE%" == "yes" Echo Making local copy of the JDK, which is required
IF "%DEBUG_UPDATE%" == "yes" Echo for PTF processing:
IF "%DEBUG_UPDATE%" == "yes" Echo JAVA_HOME: [ %JAVA_HOME% ]
IF "%DEBUG_UPDATE%" == "yes" Echo JAVA_TMP:  [ %JAVA_TMP% ]
IF "%DEBUG_UPDATE%" == "yes" Echo.
IF NOT EXIST "%JAVA_TMP%" MKDIR "%JAVA_TMP%"
IF NOT EXIST "%JAVA_TMP%" GoTo FailedCopyJDK
XCOPY "%JAVA_HOME%" "%JAVA_TMP%" /E /Q
IF NOT EXIST "%JAVA_TMP%\bin" GoTo FailedCopyJDK
IF NOT EXIST "%JAVA_TMP%\bin\java.exe" GoTo FailedCopyJDK
Goto OkCopyJDK

:SkipCopyJDK
Echo EFix processing was selected; no local copy of the
Echo JDK will be created.  PTF processing is disabled.
Echo.
goto LaunchInstaller

:FailedCopyJDK
Echo The JDK could not be copied into the current directory.
Echo JAVA_HOME: [ %JAVA_HOME% ]
Echo JAVA_TMP:  [ %JAVA_TMP% ]
Echo.
Echo Exiting.
Echo.
GOTO Usage

:OKCopyJDK
IF "%DEBUG_UPDATE%" == "yes" Echo The JDK was successfully copied.
IF "%DEBUG_UPDATE%" == "yes" Echo.
Set JAVA_HOME=%JAVA_TMP%
GoTo LaunchInstaller

:LaunchInstaller
Echo.
Echo Launching the update installer wizard.
Echo Performing dependency check and writing additional information to logs
set WindowsLAF=com.sun.java.swing.plaf.windows.WindowsLookAndFeel
set PATH=%PATH%;"%JAVA_HOME%"\bin
java.exe -DprodHelper=com.ibm.commerce.install.update.WCProductDetectHelper -Dcom.ibm.websphere.update.debug=false -Dcom.ibm.websphere.update.ptf.debug=false -Dswing.defaultlaf=%WindowsLAF% -Xmx256m -cp "%~dp0lib\wcsdepcheck.jar;%~dp0lib\WCextensions.jar;%~dp0installer.jar;%~dp0lib\extfile.jar;%~dp0lib\jtopen.jar" run

goto done

:MissingEnvVariable
echo Undefined environment variable, Please ensure that WAS_HOME and TC_ROOT is set
echo Undefined environment variable, Please ensure that WAS_HOME and TC_ROOT is set >> %LOGFILENAME% 2>&1
goto done

:IncorrectUser
echo "Incorrect userid"
echo "Incorrect userid" >> %LOGFILENAME% 2>&1
goto done

:NotEnoughAuthority
echo "Incorrect Install Authority"
echo "Incorrect Install Authority" >> %LOGFILENAME% 2>&1
goto done

:FailedToCopy
echo Copy required properties files failed
echo Copy required properties files failed >> %LOGFILENAME% 2>&1
goto done

:FailWizard
Echo The wizard jar file does not exist, exiting
Echo.
goto usage

:Usage
echo "Usage: updateWizard"
endlocal

:done
Echo Exiting, Please check %LaunchTitle%.log for more information
Echo.
Echo To complete the installation process you will need to run %TC_ROOT%\tmp\utils\postInstall.bat
Echo 
Echo.
Echo End of [ %LaunchTitle% ] script
Echo.
endlocal
